precision highp float;

//#define M_PI 3.14159
const int MAX_NUM_OF_TOUCH = 2;
// Scale up the space by  1/SPACE_DIVIDED
const float SPACE_DIVIDED = 12.0;

const float WAVE_HEIGHT = 0.007;
const float WAVE_LINE_HEIGHT = 0.0025;
//const float STEP =0.015;
const float RADIUS = 0.12;

const float RADIUS_STEP1 =0.06;
const float RADIUS_STEP2 =0.04;
//const float WAVE_TIMES = 4.0*M_PI;
const float WAVE_SPEED_FACTOR = 0.02;
const float DISTANCE_FACTOR = 3.01;
const float HALF_CIRCLE_RADiUS =0.08;

//y axis of the top line  of the keyboard
uniform lowp float sugLineY ;
uniform highp float iGlobalTime;
uniform highp vec2 iResolution;
uniform highp float dp;
uniform sampler2D inputImageTexture;
uniform highp float drawShineEffect;
uniform int currentIndex;

//uniform lowp float duration;
//uniform int currentIndex;
uniform int preIndex;
uniform int enableEffect;
uniform  vec2 touchPosition[MAX_NUM_OF_TOUCH];

varying float elapseFactor[MAX_NUM_OF_TOUCH];
varying float elapseDelayFactor[MAX_NUM_OF_TOUCH];
varying float isTouchDraw[MAX_NUM_OF_TOUCH];
varying float isTouchDelayDraw[MAX_NUM_OF_TOUCH];
//varying float waveAlphaFactor;
varying vec2 x_y_proportion;
varying vec2 textureCoordinate;

// 青蓝
vec3 cicleOutsideColor =vec3(0.0,0.645,1.00);
// 洋红
vec3 cicleInsideColor  = vec3(1.00,0.016,0.504);

// #0D0733
vec3 waveSideColor = vec3(0.051, 0.027,0.199);
// #FF4081
vec3 waveSide1Color = vec3(1.0, 0.251,0.504);
// #FFEE58
vec3 waveCenterColor = vec3(1.0, 0.93,0.344);

// wave color
vec3 waveColor =  vec3(0.598, 0.297,0.355);
//vec3 waveColor  = vec3(1.00,0.016,0.504);
vec3 waveTopColor  = vec3(0.776,0.749,0.216);
vec3 waveCircleColor  = vec3(0.0,0.768,0.875);

vec3 smallCircleInsideColor = vec3(0.051,0.035,0.192);
vec3 smallCircleOutsideColor = vec3(0.055,0.039,0.235);



vec3 circleColor(vec2 _st, float _radius){
    vec2 l = _st-vec2(0.5);
    float result = smoothstep(_radius-(_radius*0.01),
                         _radius+(_radius*0.01),
                         dot(l,l)*8.0);
    if(result==1.0){
        return smallCircleOutsideColor;
    }else{
        return smallCircleInsideColor;
    }
}

vec2 reverseYPoint(vec2 point) {
    return vec2(point.x,1.0-point.y) ;
}


vec4 bigCircleColor(vec2 curCoord,vec2 circleCenter,
            vec4 curColor,vec3 newColor,
            float cTime,float delay,float initAlpha,float step)
{
    float cRadius =cTime*RADIUS;
    float currentDistance = distance(curCoord,circleCenter);
    if (currentDistance>=(cRadius-step)&&currentDistance<=(cRadius+step)) {
        float weight=1.0;
        float deltaRadius = 1.0-abs(currentDistance-cRadius)/(step*2.0);
        if(cTime<delay*RADIUS){
            weight = initAlpha;
        }
        else {
            weight = initAlpha * (1.0-smoothstep(0.0,1.0,(cTime-delay)))*deltaRadius;
        }
        return vec4(mix(curColor.rgb,newColor,weight),1.0);
    }
    else{
        return curColor;
    }
}

vec4 touchBgColor(vec2 curCoord,vec2 circleCenter,float radius,
                        vec4 curColor,vec3 newColor,float alpha )
{
    float currentDistance = distance(curCoord,circleCenter);
    return vec4(mix(curColor.rgb,newColor,step(currentDistance,radius)*(1.0-alpha)*(1.0-currentDistance/radius)),1.0);
}


vec2 faceStretch(vec2 curCoord, vec2 originPosition, vec2 targetPosition, float radius, float curve)
{
    vec2 direction = targetPosition - originPosition;
    float lengthA = length(direction);
    if(lengthA<0.0001)   return direction;
    float lengthB = min(lengthA, radius);
    direction *= lengthB / lengthA;
    float infect = distance(curCoord, originPosition)/radius ;

    infect = clamp(1.0 - infect, 0.0, 1.0);
    infect = pow(infect, curve);
    return direction * infect;
}

bool checkNotOrignPoint(vec2 point){
    return point!=vec2(0.0,0.0);
}

void main()
{
    vec2 newTextureCoordinate = textureCoordinate*x_y_proportion;
    vec2 curCoord = newTextureCoordinate;
    vec2 newTouchPosition[MAX_NUM_OF_TOUCH];
    vec2 st = textureCoordinate;
    // Now we have SPACE_DIVIDED  spaces
    st *= vec2(x_y_proportion.x*SPACE_DIVIDED*dp,SPACE_DIVIDED*dp);
    // Wrap arround 1.0
    st = fract(st);
    vec4 finalColor = vec4(circleColor(st,0.8),1.0);

    float waveHeightFactor = 1.0;

    for(int i=0;i<MAX_NUM_OF_TOUCH;i++){
            newTouchPosition[i] = touchPosition[i]*x_y_proportion;
            // stretch wave

                // touch  effect
                if(isTouchDraw[i]==1.0&&finalColor.rgb!= smallCircleOutsideColor){
                    finalColor = bigCircleColor(newTextureCoordinate,newTouchPosition[i],finalColor,
                                                    cicleOutsideColor,elapseFactor[i],0.5,0.5,RADIUS_STEP1);
                }
                  // touch delay effect
                if(isTouchDelayDraw[i]==1.0&&finalColor.rgb!=smallCircleOutsideColor){
                       finalColor = bigCircleColor(newTextureCoordinate,newTouchPosition[i],finalColor,
                                cicleInsideColor,elapseDelayFactor[i],0.3,0.7,RADIUS_STEP2);
                }

                   // touch shine color
                 if(isTouchDraw[i]==1.0&&drawShineEffect==1.0)
                {

                     finalColor = touchBgColor(newTextureCoordinate,newTouchPosition[i],
                                                    HALF_CIRCLE_RADiUS,finalColor,
                                                    waveCircleColor,elapseFactor[i]);
                }

    }

    gl_FragColor = finalColor;
}

